#ifndef F_CPU
#warning "F_CPU war noch nicht definiert, wird nun mit 8000000 definiert"
#define F_CPU 8000000UL     /* Interner Quarz */
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>     /* in älteren avr-libc Versionen <avr/delay.h>*/
#include <avr/sleep.h>
#include <stdio.h>

#include "leds.h" //Initialisierung und Ansteuerung der LEDs
#include "uart.h" //Initialisierung und Ansteuerung des UARTs

// memory mapped io für Modemzugriffe:
//#define MODEM_BASEADR	0
//#define MODEM_BASEADR	8192		/* vielleicht besser, wenn die Modemadressen hier beginnen */
#define MODEM_BASEADR	0x2000		/* oder so */

#define MODEM (((volatile uint8_t *) MODEM_BASEADR))

volatile uint8_t Modem_INT;

///////////////////////////////////////////////////////////////////////
//
//    Die folgenden Funktionen können wegfallen, wenn Modemzugriffe
//    auf o.g. Macros umgestellt werden.
//
///////////////////////////////////////////////////////////////////////

void set_reg(uint16_t address, uint8_t Value) {

	*(volatile uint8_t *) address = Value;
}

uint8_t get_reg(uint16_t address) {

	uint8_t Value = *(volatile uint8_t *) address;
	_delay_us(20);
	return Value;
}

///////////////////////////////////////////////////////////////////////
//
//    HAUPTPROGRAMMSCHLEIFE
//
///////////////////////////////////////////////////////////////////////

int main(void) {
	uint8_t Status_1;
	uint8_t Status_2;

	XMCRA = 132;

	LED_INIT();
	usart1_init();

	DDRD &= ~_BV(0);
	EIMSK |= _BV(INT0);
	EICRA |= _BV(ISC01);

	sei();
	//Globale Interrupts einschalten

	usart1_init();

	SET_LED(RED);
	_delay_ms(300);
	RESET_LED(RED);
	printf("READY\n");

	//	SET_REG(10496, 0b01000101);//Wirkt sich aus, wie er sollte 4800
	MODEM[0x0900] = 0b01000101;//Wirkt sich aus, wie er sollte 4800
	_delay_ms(1);

	//Hier wird die grundsätzliche Initialisierung des Modems vollzogen
	//	SET_REG(17664, 0b00001000);
	MODEM[0x2500] = 0b00001000;
	set_reg(17664, 0b00000001);
	set_reg(17408, 0b11101110);
	set_reg(17664, 0b11110001);
	set_reg(19200, 0b10100001);
	set_reg(8448, 0b10000100);
	set_reg(8192, 0b10000000);
	set_reg(9216, 0b10000000);
	set_reg(8960, 0b01110101);
	set_reg(8704, 0b00110000);
	_delay_ms(10);
	set_reg(9728, 0b11000000);
	set_reg(9472, 0b10010000);
	set_reg(10240, 0b10000111);
	set_reg(9984, 0b00001100);
	_delay_ms(1);

	set_reg(17664, 0b11110101);

	while (1) //Hauptschleife
	{
		set_reg(16640, 0b00000111); //RESET
//		Status_1 = GET_REG(16640); //Auslesen
		Status_1 = MODEM[0x2100];
		_delay_us(20);
		_delay_us(10);
		set_reg(17152, 0b10000011);//Mode Reg
		_delay_ms(1);
		set_reg(16896, 0b01011100);//Hier schrauben
		_delay_ms(10);
		set_reg(16640, 0b00010000);
		_delay_us(10);

		do {
		} while (Modem_INT != 1); //Die schwierigkeit könnte hier liegen

		Modem_INT = 0;

		Status_1 = MODEM[0x2100];
		_delay_us(20);
		_delay_us(20);
		printf("/////////////////////////////////\n");
		printf("Status 1: %i\n", Status_1);
//		Status_2 = GET_REG(17152);
		Status_2 = MODEM[0x2300];
		_delay_us(20);
		_delay_us(20);
		printf("Status 2: %i\n", Status_2);

	}
	return 0;
}

ISR(INT0_vect)
{
	Modem_INT = 1;//Flag setzen
}

